(* $Id: test_ppm11.ml,v 1.10 2014/02/27 16:36:36 fclement Exp $ *)

(* essai2.ml : shows what can be done using graphics and graphps. *)

open Ugraphics;;
open Unatural;;

let sc = 1.0;;

let round f = Unatural.natural_of_float (f +. 0.5);;

let scale l = sc *. l;;
let scale_int l = round (scale l);;

let szx = scale_int 100.
and szy = scale_int 100.;;

let string_of_natural n = string_of_int (Unatural.int_of_natural n);;

Graphics.open_graph
 (Printf.sprintf " %sx%s" (string_of_natural szx) (string_of_natural szy));;

let fill_rect x y w h =
  let x = scale_int x
  and y = scale_int y
  and w = scale_int w
  and h = scale_int h in
  fill_rect x y w h
;;

let fill_poly a =
  let a = Array.map (fun (x, y) -> (scale_int x, scale_int y)) a in
  fill_poly a
;;

let fill_ellipse x y rx ry =
  let x = scale_int x
  and y = scale_int y
  and rx = scale_int rx
  and ry = scale_int ry in
  fill_ellipse x y rx ry
;;

let fill_circle x y r =
  let x = scale_int x
  and y = scale_int y
  and r = scale_int r in
  fill_circle x y r
;;

let grad_rect gm x y w h =
  let x = scale_int x
  and y = scale_int y
  and w = scale_int w
  and h = scale_int h in
  Gradient.grad_rectangle gm x y w h
;;

let grad_circle gm x y r =
  let x = scale_int x
  and y = scale_int y
  and r = scale_int r in
  Gradient.grad_circle gm x y r
;;

let rgb_int r g b =
  Ugraphics.rgb (char_of_int r) (char_of_int g) (char_of_int b)
;;

let red1 = rgb_int 245 191 191
and yellow1 = rgb_int 230 230 120
and green1 = rgb_int 135 222 135
and green2 = rgb_int 40 180 80
and blue1 = rgb_int 70 135 200
;;

let background = red1
and rectangle_color = yellow1
and triangle_color = blue1
and torus_color = green2
;;

(* Was
let rgb_of_color = Colors.rgb_of_color;;
*)
let rgb_of_color = Ugraphics.rgb_of_color
;;

let perturbate c pr pg pb =
  let (r, g, b) = rgb_of_color c in
  let new_color_channel p cc =
    let cc = int_of_char cc in
    char_of_int (Pervasives.( + ) cc p) in
  let nr = new_color_channel pr r in
  let ng = new_color_channel pg g in
  let nb = new_color_channel pb b in
  
  rgb nr ng nb
;;

let perturbate_uniform c p = perturbate c p p p;;

let rectangle_perturbation_color =
  perturbate_uniform rectangle_color 20
and triangle_perturbation_color =
  perturbate_uniform triangle_color 12
and torus_perturbation_color1 =
  perturbate_uniform torus_color (-30)
and torus_perturbation_color2 =
  perturbate_uniform torus_color 30
;;

(* Background. *)
set_color background;;
fill_rect 0. 0. (float_of_natural szx) (float_of_natural szy);;

(* A rectangle with an elliptic hole. *)
set_color rectangle_color;;
fill_rect 28. 15. 42. 32.;;
set_color background;;
fill_ellipse 49. 26. 14.5 12.;;

(* A triangle. *)
set_color triangle_color;;
fill_poly [| (55., 79.); (84., 79.); (84., 50.); (55., 79.); |];;

(* A gradient-colored torus. *)
grad_circle
  (Gradient.Ellipse_Vertical
    (red, blue))
(*    (torus_perturbation_color1, torus_perturbation_color2))*)
  31. 69. 17.;;
set_color background;;
(*fill_circle 31. 69. 7.;;*)

Grimage.save_screen_ppm "test11.ppm";;
