(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_structural_mapping.mli,v 1.12 2014-02-26 23:33:54 fclement Exp $ *)

(** {3 Matrix structural mappings} *)

type 'a mapping = 'a Umatrix_graph.t
and 'a sub_mapping = 'a Umatrix_graph.t
;;

type 'a partition = 'a sub_mapping array;;

type block_number = Udelayed_dgraph.arrow_number;;

type ('a, 'b) block = ('a sub_mapping, 'b) Udelayed_dgraph.delayed_arrow;;

type ('a, 'b, 'c) t = ('a sub_mapping, 'b, 'c) Udelayed_dgraph.t;;
(** Blocks are meant to be supported by disjoint sub-domains, and the
 disjoint union of all sub-domains is meant to be the full universe. *)

val make :
  ('c -> block_number -> 'a sub_mapping -> 'b) -> 'a mapping -> ('a, 'b, 'c) t
;;

val same : ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool;;

(** {6 Access functions} *)

val get : ('a, 'b, 'c) t -> block_number -> ('a, 'b) block;;
val set_next : ('a, 'b, 'c) t -> 'a sub_mapping -> ('a, 'b, 'c) t;;
val set : ('a, 'b, 'c) t -> block_number -> 'a sub_mapping -> ('a, 'b, 'c) t;;

val last_block_number : ('a, 'b, 'c) t -> block_number;;

(** {6 Converters} *)

val partition_of_mapping : 'a partition -> 'b mapping -> 'b partition;;

val partition_to_geometry_and_values :
  'a partition -> Unatural.t array * 'a array
;;

val one_zone_partition : Umatrix_nonempty_set.size -> 'a -> 'a partition;;

val to_mapping : ('a, 'b, 'c) t -> 'a mapping;;

val to_array :
  ('a, 'b, 'c) t -> ('a sub_mapping, 'b) Udynamic_graph.arrow array
;;

val of_partition :
  ('c -> block_number -> 'a sub_mapping -> 'b) ->
  'a partition -> ('a, 'b, 'c) t
;;

val to_partition : ('a, 'b, 'c) t -> 'a partition;;

(** {6 Iterators} *)

val iter :
  (block_number -> ('a, 'b) block -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val fold_left :
  ('accu -> block_number -> ('a, 'b) block -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;

val partition_iter :
  (block_number -> 'a sub_mapping -> unit) ->
  'a partition -> unit
;;

val iter_to_do :
  (block_number -> 'a sub_mapping -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val iter_value :
  (block_number -> 'a sub_mapping -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val fold_left_value :
  ('accu -> block_number -> 'a sub_mapping -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;
